#include <util/delay.h>
#include "common.h"

#define WAIT1 10
#define WAIT2 20

t_PsxController psx_data[2];

static void sendrecv(unsigned char s, unsigned char *r1, unsigned char *r2);

/*
  Initialization
*/
void inDecoderInit(void) {
	PORTB = 0xff;
	DDRB = 0xeb;
	PORTC = 0xff;
	DDRC = 0xeb;
	TCCR2A |= _BV(WGM21);
	TCCR2B |= (_BV(CS22) | _BV(CS21) | _BV(CS20));
	OCR2A = 0x6A;
	TIFR2 |= _BV(OCF2A);
}

/*
  Check of buttons
*/
void inDecoderPoll(void) {
	unsigned char analog1;
	unsigned char analog2;
	unsigned char d1, d1a, d1b, d1c, d1d, d1e, d1f;
	unsigned char d2, d2a, d2b, d2c, d2d, d2e, d2f;

	if (bit_is_clear(TIFR2,OCF2A)) return;
	TIFR2 |= _BV(OCF2A);

	PORTB &= ~_BV(SEL1);
	PORTC &= ~_BV(SEL2);

	/* 1st byte*/
	sendrecv(0x01, &d1, &d2);

	/* 2nd byte */
	sendrecv(0x42, &d1, &d2);
	analog1 = bit_is_set(d1, 3) ? 0xff : bit_is_set(d1, 1) ? 1 : 0;
	analog2 = bit_is_set(d2, 3) ? 0xff : bit_is_set(d2, 1) ? 1 : 0;

	/* 3rd byte */
	sendrecv(0x00, &d1, &d2);

	/* 4-5th byte */
	sendrecv(0x00, &d1a, &d2a);
	sendrecv(0x00, &d1b, &d2b);

	/* 6-7th byte */
	sendrecv(0x00, &d1c, &d2c);
	sendrecv(0x00, &d1d, &d2d);

	/* 8-9th byte */
	sendrecv(0x00, &d1e, &d2e);
	sendrecv(0x00, &d1f, &d2f);

	PORTB |= _BV(SEL1);
	PORTC |= _BV(SEL2);
	PORTB |= _BV(CMD1);
	PORTC |= _BV(CMD2);

	if (analog1 == 0) {
		d1c = d1d = 0x7f;
		d1e = bit_is_clear(d1a, 7) ? 0x00 : bit_is_clear(d1a, 5) ? 0xff : 0x7f;
		d1f = bit_is_clear(d1a, 4) ? 0x00 : bit_is_clear(d1a, 6) ? 0xff : 0x7f;
	} else if (analog1 == 0xff) {
		d1a = d1b = 0xff;
		d1c = d1d = d1e = d1f = 0x7f;
	}
	if (analog2 == 0) {
		d2c = d2d = 0x7f;
		d2e = bit_is_clear(d2a, 7) ? 0x00 : bit_is_clear(d2a, 5) ? 0xff : 0x7f;
		d2f = bit_is_clear(d2a, 4) ? 0x00 : bit_is_clear(d2a, 6) ? 0xff : 0x7f;
	} else if (analog2 == 0xff) {
		d2a = d2b = 0xff;
		d2c = d2d = d2e = d2f = 0x7f;
	}

	psx_data[0].x = d1e;
	psx_data[0].y = d1f;
	psx_data[0].z = d1c;
	psx_data[0].p = d1d;
	psx_data[0].u.buttons = ~d1a;
	psx_data[0].w.but = ~d1b;
	psx_data[1].x = d2e;
	psx_data[1].y = d2f;
	psx_data[1].z = d2c;
	psx_data[1].p = d2d;
	psx_data[1].u.buttons = ~d2a;
	psx_data[1].w.but = ~d2b;
}

static void sendrecv(unsigned char s, unsigned char *r1, unsigned char *r2) {
	unsigned char i;
	unsigned char t1 = 0;
	unsigned char t2 = 0;

	for (i = 0; i < 8; i++) {
		PORTB &= ~_BV(CLK1);
		PORTC &= ~_BV(CLK2);
		if (bit_is_set(s, 0)) {
			PORTB |= _BV(CMD1);
			PORTC |= _BV(CMD2);
		} else {
			PORTB &= ~_BV(CMD1);
			PORTC &= ~_BV(CMD2);
		}
		s >>= 1;
		_delay_us(WAIT1);
		PORTB |= _BV(CLK1);
		PORTC |= _BV(CLK2);

		t1 >>= 1;
		t2 >>= 1;
		t1 |= bit_is_set(PINB, DAT1) ? 0x80 : 0;
		t2 |= bit_is_set(PINC, DAT2) ? 0x80 : 0;
		_delay_us(WAIT1);
	}
	_delay_us(WAIT2);

	*r1 = t1;
	*r2 = t2;

	return;
}
